@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #if HAVE_CONFIG_H
@@ -81,10 +81,6 @@ static int muscle_match_card(sc_card_t *card)
 	u8 response[64];
 	int r;
 
-	/* Since we send an APDU, the card's logout function may be called...
-	 * however it's not always properly nulled out... */
-	card->ops->logout = NULL;
-
 	if (msc_select_applet(card, muscleAppletId, sizeof muscleAppletId) == 1) {
 		/* Muscle applet is present, check the protocol version to be sure */
 		sc_format_apdu(card, &apdu, SC_APDU_CASE_2, 0x3C, 0x00, 0x00);
@@ -153,7 +149,7 @@ static int muscle_create_directory(sc_card_t *card, sc_file_t *file)
 	u8* oid = objectId.id;
 	unsigned id = file->id;
 	unsigned short read_perm = 0, write_perm = 0, delete_perm = 0;
-	int objectSize;
+	size_t objectSize;
 	int r;
 	if(id == 0) /* No null name files */
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -178,7 +174,7 @@ static int muscle_create_directory(sc_card_t *card, sc_file_t *file)
 static int muscle_create_file(sc_card_t *card, sc_file_t *file)
 {
 	mscfs_t *fs = MUSCLE_FS(card);
-	int objectSize = file->size;
+	size_t objectSize = file->size;
 	unsigned short read_perm = 0, write_perm = 0, delete_perm = 0;
 	msc_id objectId;
 	int r;
@@ -198,7 +194,7 @@ static int muscle_create_file(sc_card_t *card, sc_file_t *file)
 	return r;
 }
 
-static int muscle_read_binary(sc_card_t *card, unsigned int idx, u8* buf, size_t count, unsigned long flags)
+static int muscle_read_binary(sc_card_t *card, unsigned int idx, u8* buf, size_t count, unsigned long *flags)
 {
 	mscfs_t *fs = MUSCLE_FS(card);
 	int r;
@@ -240,7 +236,7 @@ static int muscle_update_binary(sc_card_t *card, unsigned int idx, const u8* buf
 		oid[2] = oid[3] = 0;
 	}
 	if(file->size < idx + count) {
-		int newFileSize = idx + count;
+		size_t newFileSize = idx + count;
 		u8* buffer = malloc(newFileSize);
 		if(buffer == NULL) LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
 
@@ -278,9 +274,10 @@ static int muscle_delete_mscfs_file(sc_card_t *card, mscfs_file_t *file_data)
 		int x;
 		mscfs_file_t *childFile;
 		/* Delete children */
-		mscfs_check_cache(fs);
+		r = mscfs_check_cache(fs);
+		if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			"DELETING Children of: %02X%02X%02X%02X\n",
 			oid[0],oid[1],oid[2],oid[3]);
 		for(x = 0; x < fs->cache.size; x++) {
@@ -289,7 +286,7 @@ static int muscle_delete_mscfs_file(sc_card_t *card, mscfs_file_t *file_data)
 			objectId = childFile->objectId;
 
 			if(0 == memcmp(oid + 2, objectId.id, 2) && !childFile->deleteFile) {
-				sc_log(card->ctx, 
+				sc_log(card->ctx,
 					"DELETING: %02X%02X%02X%02X\n",
 					objectId.id[0],objectId.id[1],
 					objectId.id[2],objectId.id[3]);
@@ -372,12 +369,13 @@ static int select_item(sc_card_t *card, const sc_path_t *path_in, sc_file_t ** f
 {
 	mscfs_t *fs = MUSCLE_FS(card);
 	mscfs_file_t *file_data = NULL;
-	int pathlen = path_in->len;
+	size_t pathlen = path_in->len;
 	int r = 0;
 	int objectIndex;
 	u8* oid;
 
-	mscfs_check_cache(fs);
+	r = mscfs_check_cache(fs);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 	r = mscfs_loadFileInfo(fs, path_in->value, path_in->len, &file_data, &objectIndex);
 	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,r);
 
@@ -524,16 +522,17 @@ static int muscle_list_files(sc_card_t *card, u8 *buf, size_t bufLen)
 {
 	muscle_private_t* priv = MUSCLE_DATA(card);
 	mscfs_t *fs = priv->fs;
-	int x;
+	int x, r;
 	int count = 0;
 
-	mscfs_check_cache(priv->fs);
+	r = mscfs_check_cache(priv->fs);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 
 	for(x = 0; x < fs->cache.size; x++) {
 		u8* oid = fs->cache.array[x].objectId.id;
 		if (bufLen < 2)
 			break;
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			"FILE: %02X%02X%02X%02X\n",
 			oid[0],oid[1],oid[2],oid[3]);
 		if(0 == memcmp(fs->currentPath, oid, 2)) {
@@ -853,6 +852,19 @@ static int muscle_card_reader_lock_obtained(sc_card_t *card, int was_reset)
 	LOG_FUNC_RETURN(card->ctx, r);
 }
 
+static int muscle_logout(sc_card_t *card)
+{
+	int r = SC_ERROR_NOT_SUPPORTED;
+
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
+
+	if (msc_select_applet(card, muscleAppletId, sizeof muscleAppletId) == 1) {
+		r = SC_SUCCESS;
+	}
+
+	LOG_FUNC_RETURN(card->ctx, r);
+}
+
 
 static struct sc_card_driver * sc_get_driver(void)
 {
@@ -881,6 +893,7 @@ static struct sc_card_driver * sc_get_driver(void)
 	muscle_ops.delete_file = muscle_delete_file;
 	muscle_ops.list_files = muscle_list_files;
 	muscle_ops.card_reader_lock_obtained = muscle_card_reader_lock_obtained;
+	muscle_ops.logout = muscle_logout;
 
 	return &muscle_drv;
 }
