@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #if HAVE_CONFIG_H
@@ -126,7 +126,7 @@ int sc_restore_security_env(sc_card_t *card, int se_num)
 	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 }
 
-int sc_verify(sc_card_t *card, unsigned int type, int ref, 
+int sc_verify(sc_card_t *card, unsigned int type, int ref,
 	      const u8 *pin, size_t pinlen, int *tries_left)
 {
 	struct sc_pin_cmd_data data;
@@ -335,7 +335,7 @@ int sc_build_pin(u8 *buf, size_t buflen, struct sc_pin_cmd_pin *pin, int pad)
 		}
 	}
 
-	return i;
+	return (int)i;
 }
 
 int
