@@ -161,10 +161,10 @@ static char * get_pin(struct sc_pkcs15_object *obj)
 	}
 }
 
-static int read_input(u8 *buf, int buflen)
+static size_t read_input(u8 *buf, int buflen)
 {
 	FILE *inf;
-	int c;
+	size_t c;
 
 	if (opt_input==NULL) {
 		inf = stdin;
@@ -172,21 +172,21 @@ static int read_input(u8 *buf, int buflen)
 		inf = fopen(opt_input, "rb");
 		if (inf == NULL) {
 			fprintf(stderr, "Unable to open '%s' for reading.\n", opt_input);
-			return -1;
+			return 0;
 		}
 	}
 	c = fread(buf, 1, buflen, inf);
 	if (inf!=stdin) {
 		fclose(inf);
 	}
-	if (c < 0) {
+	if (c <= 0) {
 		perror("read");
-		return -1;
+		return 0;
 	}
 	return c;
 }
 
-static int write_output(const u8 *buf, int len)
+static int write_output(const u8 *buf, size_t len)
 {
 	FILE *outf;
 	int output_binary = (opt_output == NULL && opt_raw == 0 ? 0 : 1);
@@ -215,18 +215,19 @@ static int sign(struct sc_pkcs15_object *obj)
 {
 	u8 buf[1024], out[1024];
 	struct sc_pkcs15_prkey_info *key = (struct sc_pkcs15_prkey_info *) obj->data;
-	int r, c, len;
+	int r;
+	size_t c, len;
 
 	if (opt_input == NULL) {
 		fprintf(stderr, "No input file specified. Reading from stdin\n");
 	}
 
 	c = read_input(buf, sizeof(buf));
-	if (c < 0)
+	if (c <= 0)
 		return 2;
 	len = sizeof(out);
 	if (obj->type == SC_PKCS15_TYPE_PRKEY_RSA
-			&& !(opt_crypt_flags & SC_ALGORITHM_RSA_PAD_PKCS1)
+			&& !(opt_crypt_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_01)
 			&& (size_t)c != key->modulus_length/8) {
 		fprintf(stderr, "Input has to be exactly %lu bytes, when using no padding.\n",
 			(unsigned long) key->modulus_length/8);
@@ -269,13 +270,14 @@ static int sign(struct sc_pkcs15_object *obj)
 static int decipher(struct sc_pkcs15_object *obj)
 {
 	u8 buf[1024], out[1024];
-	int r, c, len;
+	int r, len;
+	size_t c;
 
 	if (opt_input == NULL) {
 		fprintf(stderr, "No input file specified. Reading from stdin\n");
 	}
 	c = read_input(buf, sizeof(buf));
-	if (c < 0)
+	if (c <= 0)
 		return 2;
 
 	len = sizeof(out);
@@ -284,7 +286,7 @@ static int decipher(struct sc_pkcs15_object *obj)
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
-	r = sc_pkcs15_decipher(p15card, obj, opt_crypt_flags & SC_ALGORITHM_RSA_PAD_PKCS1, buf, c, out, len, NULL);
+	r = sc_pkcs15_decipher(p15card, obj, opt_crypt_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_02, buf, c, out, len, NULL);
 	if (r < 0) {
 		fprintf(stderr, "Decrypt failed: %s\n", sc_strerror(r));
 		return 1;
@@ -343,12 +345,12 @@ static int get_key(unsigned int usage, sc_pkcs15_object_t **result)
 			free(pincode);
 			return 5;
 		}
-		
+
 		/*
 		 * Do what PKCS#11 would do for keys requiring CKA_ALWAYS_AUTHENTICATE
-		 * and CKU_CONTEXT_SPECIFIC login to let driver know this verify will be followed by 
+		 * and CKU_CONTEXT_SPECIFIC login to let driver know this verify will be followed by
 		 * a crypto operation.  Card drivers can test for SC_AC_CONTEXT_SPECIFIC
-		 * to do any special handling. 
+		 * to do any special handling.
 		 */
 		if (key->user_consent && pin) {
 			int auth_meth_saved;
