@@ -81,6 +81,7 @@ enum {
 	OPT_PASSWORD_SHARES_TOTAL
 };
 
+// clang-format off
 static const struct option options[] = {
 	{ "initialize",				0, NULL,		'X' },
 	{ "create-dkek-share",		1, NULL,		'C' },
@@ -112,6 +113,7 @@ static const struct option options[] = {
 	{ "verbose",				0, NULL,		'v' },
 	{ NULL, 0, NULL, 0 }
 };
+// clang-format on
 
 static const char *option_help[] = {
 	"Initialize token",
@@ -735,7 +737,7 @@ static int recreate_password_from_shares(char **pwd, int *pwdlen, int num_of_pas
 	}
 	binlen = 64;
 	sc_hex_to_bin(inbuf, bin, &binlen);
-	BN_bin2bn(bin, binlen, prime);
+	BN_bin2bn(bin, (int)binlen, prime);
 
 	sp = shares;
 	for (i = 0; i < num_of_password_shares; i++) {
@@ -770,7 +772,7 @@ static int recreate_password_from_shares(char **pwd, int *pwdlen, int num_of_pas
 		}
 		binlen = 64;
 		sc_hex_to_bin(inbuf, bin, &binlen);
-		BN_bin2bn(bin, binlen, (sp->y));
+		BN_bin2bn(bin, (int)binlen, (sp->y));
 
 		sp++;
 	}
@@ -844,7 +846,7 @@ static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const c
 		if (num_of_password_shares == -1) {
 			printf("Enter password to decrypt DKEK share : ");
 			util_getpass(&pwd, NULL, stdin);
-			pwdlen = strlen(pwd);
+			pwdlen = (int)strlen(pwd);
 			printf("\n");
 		} else {
 			r = recreate_password_from_shares(&pwd, &pwdlen, num_of_password_shares);
@@ -855,7 +857,7 @@ static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const c
 
 	} else {
 		pwd = (char *) password;
-		pwdlen = strlen(password);
+		pwdlen = (int)strlen(password);
 	}
 
 	printf("Deciphering DKEK share, please wait...\n");
@@ -905,7 +907,7 @@ static int import_dkek_share(sc_card_t *card, const char *inf, int iter, const c
 
 static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const char *password, int num_of_password_shares)
 {
-	// hex output can be used in the SCSH shell with the 
+	// hex output can be used in the SCSH shell with the
 	// decrypt_keyblob.js file
 	sc_cardctl_sc_hsm_dkek_t dkekinfo;
 	EVP_CIPHER_CTX *bn_ctx = NULL;
@@ -945,7 +947,7 @@ static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const ch
 		if (num_of_password_shares == -1) {
 			printf("Enter password to decrypt DKEK share : ");
 			util_getpass(&pwd, NULL, stdin);
-			pwdlen = strlen(pwd);
+			pwdlen = (int)strlen(pwd);
 			printf("\n");
 		} else {
 			r = recreate_password_from_shares(&pwd, &pwdlen, num_of_password_shares);
@@ -956,7 +958,7 @@ static int print_dkek_share(sc_card_t *card, const char *inf, int iter, const ch
 
 	} else {
 		pwd = (char *) password;
-		pwdlen = strlen(password);
+		pwdlen = (int)strlen(password);
 	}
 
 	printf("Deciphering DKEK share, please wait...\n");
@@ -1037,7 +1039,7 @@ static void ask_for_password(char **pwd, int *pwdlen)
 			printf("Passwords do not match. Please retry.\n");
 			continue;
 		}
-		*pwdlen = strlen(*pwd);
+		*pwdlen = (int)strlen(*pwd);
 		break;
 	}
 
@@ -1206,7 +1208,7 @@ static int create_dkek_share(sc_card_t *card, const char *outf, int iter, const
 
 	} else {
 		pwd = (char *) password;
-		pwdlen = strlen(password);
+		pwdlen = (int)strlen(password);
 	}
 
 	if (r < 0) {
@@ -1402,7 +1404,7 @@ static int wrap_key(sc_context_t *ctx, sc_card_t *card, int keyid, const char *o
 			fprintf(stderr, "Error reading PRKD file %s. Skipping.\n", sc_strerror(ef_prkd_len));
 			ef_prkd_len = 0;
 		} else {
-			ef_prkd_len = determineLength(ef_prkd, ef_prkd_len);
+			ef_prkd_len = (int)determineLength(ef_prkd, ef_prkd_len);
 		}
 	}
 
@@ -1421,7 +1423,7 @@ static int wrap_key(sc_context_t *ctx, sc_card_t *card, int keyid, const char *o
 			fprintf(stderr, "Error reading certificate %s. Skipping\n", sc_strerror(ef_cert_len));
 			ef_cert_len = 0;
 		} else {
-			ef_cert_len = determineLength(ef_cert, ef_cert_len);
+			ef_cert_len = (int)determineLength(ef_cert, ef_cert_len);
 		}
 	}
 
@@ -1528,8 +1530,10 @@ static int unwrap_key(sc_card_t *card, int keyid, const char *inf, const char *p
 	u8 fid[2];
 	char *lpin = NULL;
 	unsigned int cla, tag;
-	int r, keybloblen;
+	int r;
+	size_t keybloblen;
 	size_t len, olen, prkd_len, cert_len;
+	ssize_t sz;
 
 	if ((keyid < 1) || (keyid > 255)) {
 		fprintf(stderr, "Invalid key reference (must be 0 < keyid <= 255)\n");
@@ -1548,12 +1552,13 @@ static int unwrap_key(sc_card_t *card, int keyid, const char *inf, const char *p
 		return -1;
 	}
 
-	keybloblen = fread(keyblob, 1, sizeof(keyblob), in);
+	sz = fread(keyblob, 1, sizeof(keyblob), in);
 	fclose(in);
-	if (keybloblen < 0) {
+	if (sz < 0) {
 		perror(inf);
 		return -1;
 	}
+	keybloblen = sz;
 
 	ptr = keyblob;
 	if ((sc_asn1_read_tag(&ptr, keybloblen, &cla, &tag, &len) != SC_SUCCESS)
@@ -1703,7 +1708,8 @@ static int export_key(sc_card_t *card, int keyid, const char *outf)
 	u8 dev_aut_cert[MAX_CERT];
 	u8 dica[MAX_CERT];
 	u8 tag = SC_ASN1_TAG_CONSTRUCTED | SC_ASN1_TAG_SEQUENCE; /* 0x30 */
-	int r = 0, ef_cert_len, dev_aut_cert_len, dica_len, total_certs_len;
+	int r = 0, ef_cert_len, total_certs_len;
+	size_t dev_aut_cert_len, dica_len;
 	u8 *data = NULL, *out = NULL, *ptr;
 	size_t datalen, outlen;
 
@@ -1729,7 +1735,7 @@ static int export_key(sc_card_t *card, int keyid, const char *outf)
 		fprintf(stderr, "Error reading certificate %s. Skipping\n", sc_strerror(ef_cert_len));
 		ef_cert_len = 0;
 	} else {
-		ef_cert_len = determineLength(ef_cert, ef_cert_len);
+		ef_cert_len = (int)determineLength(ef_cert, ef_cert_len);
 	}
 
 	/* C_DevAut */
@@ -1949,10 +1955,10 @@ int main(int argc, char *argv[])
 			action_count++;
 			break;
 		case 'K':
-			opt_num_of_pub_keys = atol(optarg);
+			opt_num_of_pub_keys = (int)atol(optarg);
 			break;
 		case 'n':
-			opt_required_pub_keys = atol(optarg);
+			opt_required_pub_keys = (int)atol(optarg);
 			break;
 		case 'e':
 			do_export_key = 1;
@@ -1978,7 +1984,7 @@ int main(int argc, char *argv[])
 			util_get_pin(optarg, &opt_pin);
 			break;
 		case OPT_RETRY:
-			opt_retry_counter = atol(optarg);
+			opt_retry_counter = (int)atol(optarg);
 			break;
 		case OPT_BIO1:
 			opt_bio1 = optarg;
@@ -1987,19 +1993,19 @@ int main(int argc, char *argv[])
 			opt_bio2 = optarg;
 			break;
 		case OPT_PASSWORD_SHARES_THRESHOLD:
-			opt_password_shares_threshold = atol(optarg);
+			opt_password_shares_threshold = (int)atol(optarg);
 			break;
 		case OPT_PASSWORD_SHARES_TOTAL:
-			opt_password_shares_total = atol(optarg);
+			opt_password_shares_total = (int)atol(optarg);
 			break;
 		case 's':
-			opt_dkek_shares = atol(optarg);
+			opt_dkek_shares = (int)atol(optarg);
 			break;
 		case 'f':
 			opt_force = 1;
 			break;
 		case 'i':
-			opt_key_reference = atol(optarg);
+			opt_key_reference = (int)atol(optarg);
 			break;
 		case 'r':
 			opt_reader = optarg;
